/**

Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

This file is part of Spago.

Spago is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
any later version.

Spago is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with Spago; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/

package it.eng.spago.tags;

import java.util.ArrayList;
import java.util.List;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.RequestContainerAccess;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.ResponseContainerAccess;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.ContextScooping;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;
import it.eng.spago.message.MessageBundle;

/**
 * @author Butano
 */
public class MessageTag extends TagSupport {
	/**
     * 
     */
    private static final long serialVersionUID = 1L;
    private String _code = null;
	private String _bundle = null;
	private String _paramsName = null;
	private String _paramsScope = Constants.SERVICE_REQUEST;
	
	public MessageTag() {
		super();
	}
	
	public int doStartTag() throws JspException {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION, "MessageTag::doStartTag:: invocato");        
        String message = null;
        HttpServletRequest httpRequest = (HttpServletRequest)pageContext.getRequest();
        RequestContainer requestContainer = RequestContainerAccess.getRequestContainer(httpRequest);
        ResponseContainer responseContainer = ResponseContainerAccess.getResponseContainer(httpRequest);

        Object params = ContextScooping.getScopedParameter(
    			requestContainer, responseContainer, this._paramsName,
    			_paramsScope, null);
    	
    	List paramsList = getParamsAsList(params);
        
        if (_bundle != null)
        	message = MessageBundle.getMessage(_code, _bundle, paramsList);
        else
        	message = MessageBundle.getMessage(_code, paramsList);
        StringBuffer htmlStream = new StringBuffer();
        htmlStream.append(message);        
        try {
            pageContext.getOut().print(htmlStream);
        } 
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                "MessageTag::doStartTag:: Impossibile inviare lo stream");
            throw new JspException("Impossibile inviare lo stream");
        } 
        return SKIP_BODY;
	}
	
	private List getParamsAsList(Object params) {
    	List paramsList = null;
    	if (params == null) {
    		paramsList = new ArrayList();
    	} else {
    		if (params instanceof List) {
	    		paramsList = (List)params;
	    	} else {
	    		paramsList = new ArrayList(1);
	    		if (params instanceof String) {
	    			paramsList.add(params);
	    		} else {
	    			paramsList.add(params.toString());
	    		}
	    	}
    	}
    	return paramsList;
	}
	
	public void setCode(String code) {
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "MessageTag::setCode: code [" + code + "]");
        _code = code;
    }

	public String getCode() {
        return _code;
    }
	
	public void setBundle(String bundle) {
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "MessageTag::setBundle: bundle [" + bundle + "]");
        _bundle = bundle;
    }
	
	public String getBundlde() {
        return _bundle;
    }
			
    public int doEndTag() throws JspException {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION, "MessageTag::doEndTag:: invocato");
        return super.doEndTag();
    }

	public String getParamsName() {
		return _paramsName;
	}

	public void setParamsName(String _params) {
		this._paramsName = _params;
	}

	public String getParamsScope() {
		return _paramsScope;
	}

	public void setParamsScope(String scope) {
		_paramsScope = scope;
	} 	
}
