package it.eng.console.attachment.action;

import it.eng.console.audit.bo.ProcessInstanceBO;
import it.eng.console.util.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.dispatching.action.AbstractHttpAction;
import it.eng.spago.message.MessageBundle;
import it.eng.spago.tracing.TracerSingleton;

import java.io.InputStream;

import javax.activation.DataHandler;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.NormalizedMessage;
import javax.servlet.ServletOutputStream;

import org.spagic.metadb.dbutils.serialization.CalendarConverter;
import org.spagic.metadb.model.TransitionInstance;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.DomDriver;

public class OpenAttachment extends AbstractHttpAction {

	public void service(SourceBean serviceRequest, SourceBean serviceResponse)
			throws Exception {
		// TODO Auto-generated method stub
		int indexAttachment = Integer.parseInt((String)serviceRequest.getAttribute("indexAttachment"));
		long idJournal = Long.parseLong((String)serviceRequest.getAttribute("idJournal"));
		String direction = (String)serviceRequest.getAttribute("direction");
		NormalizedMessage message = null;
		TransitionInstance transitionInstance = ProcessInstanceBO.getTransitionInstance(idJournal);
		//if direction IN we should retrieve the normalized message from the messageExchange 
		//saved in the transictiondata field
		//if direction is OUT we should retrieve message directly in the transitionDataOut field
		if(direction.equalsIgnoreCase("in")) {
			MessageExchange messageExchange = deserializeMessageExchange(transitionInstance.getTransitionData());
			message = messageExchange.getMessage(direction);
		} else 
			message = (NormalizedMessage)deserializeNormalizedMessage(transitionInstance.getTransitionDataOut());
		
		
		if (message != null) {
			Object[] attachmentNames = message.getAttachmentNames().toArray();
			String attachmentName = (String)attachmentNames[indexAttachment];
			DataHandler attachment = message.getAttachment(attachmentName);
			String contentType = attachment.getContentType();
			getHttpResponse().setContentType(contentType);
			getHttpResponse().setHeader("Content-Disposition","attachment; filename=\"" + MessageBundle.getMessage("attachment", "properties.spagic") + " " + (indexAttachment+1) + "\";");
			InputStream attachmentInputStream = attachment.getInputStream();
			ServletOutputStream outputStream = getHttpResponse().getOutputStream();
			byte[] buffer = new byte[1024];
			int byteLetti;
			while((byteLetti = attachmentInputStream.read(buffer)) != -1)
				outputStream.write(buffer, 0, byteLetti);
			outputStream.flush();
			outputStream.close();
			attachmentInputStream.close();
		}
		freezeHttpResponse();

	}
	
	private MessageExchange deserializeMessageExchange(String xml) {
		MessageExchange exchange = null;
		XStream xstream = new XStream(new DomDriver());
		xstream.alias("messageExchange", MessageExchange.class);
		xstream.registerConverter(new CalendarConverter());
		if (!(xml.length()>0)) {
			return null;
		}
        
		try {
			// Deserialize from a String
			exchange = (MessageExchange)xstream.fromXML(xml);
		} catch (Exception ex) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "deserialize: ", ex );
		}
        return exchange;
	}
	
	
	private Object deserializeNormalizedMessage(String inputString){
		NormalizedMessage message = null;
		XStream xstream = new XStream(new DomDriver());
		xstream.alias("NormalizedMessage", NormalizedMessage.class);
		xstream.registerConverter(new CalendarConverter());
		if (!(inputString.length()>0)) {
			return null;
		}       
		try {
			// Deserialize from a String
			message = (NormalizedMessage)xstream.fromXML(inputString);
		} catch (Exception ex) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "deserialize: ", ex );
		}
		return message;
	}

}
