/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.audit.action;

import it.eng.console.process.bo.ProcessBO;
import it.eng.console.query.bo.QueryBO;
import it.eng.console.util.Constants;
import it.eng.console.util.CronExpressionUtil;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.dispatching.action.AbstractAction;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago.tracing.TracerSingleton;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import org.spagic.components.jobs.backupcfg.ConfigureAutomaticBackupStart_v_0Proxy;
import org.spagic.components.jobs.backupcfg.parameters.ParametersType;
import org.spagic.metadb.base.ProcessInstanceStateConstants;
import org.spagic.metadb.model.SearchQuery;

public class ProcessSearchAction extends AbstractAction {

	public void service(SourceBean serviceRequest, SourceBean serviceResponse)
			throws Exception {
		 
		SourceBean queryBeanList = new SourceBean("LIST");
		
		List<SearchQuery> queryList = QueryBO.getQueryList();
		//create response
		SearchQuery query = null;
		for(int indexQuery=0; indexQuery<queryList.size(); indexQuery++) {
			query = (SearchQuery)queryList.get(indexQuery);
			if (query!=null) {
				queryBeanList.setAttribute(createQueryBean(query));
			}
		}
		serviceResponse.setAttribute(queryBeanList);
		

		List processList = ProcessBO.getProcessListOrderByName();
		serviceResponse.setAttribute("processo", processList);
		if (serviceRequest.getAttribute(Constants.LBL_BACKUP) != null) {
			ConfigureAutomaticBackupStart_v_0Proxy cfgBackupProxy = new ConfigureAutomaticBackupStart_v_0Proxy();
			String locationCfgBackupWS = (String)ConfigSingleton.getInstance().getAttribute("CONSOLE.locationCfgBackupWS");
			cfgBackupProxy.setEndpoint(locationCfgBackupWS);
			ParametersType input = new ParametersType();
			input.setSet_cfg("false");
			try{
				ParametersType output = cfgBackupProxy.run(input);
				String cronExpression = output.getFrequency();
				serviceResponse.setAttribute(Constants.LBL_FREQUENCY, CronExpressionUtil.parse(cronExpression));
				serviceResponse.setAttribute(Constants.LBL_EXPIRED_TIME, output.getExpired_time());
				serviceResponse.setAttribute(Constants.LBL_AUTOBACKUP, output.getAuto_backup());
			}catch(RemoteException re) {
				TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessSearchAction::service:: ", re);
				getErrorHandler().addError(new EMFUserError(EMFErrorSeverity.INFORMATION, "10016", "properties.console.messages"));
				return;
			}
		} else {
			List processStateList = new ArrayList();
			if (serviceRequest.getAttribute(Constants.LBL_RESTART) == null || serviceRequest.getAttribute(Constants.LBL_RESTART).equals("FALSE")) {
				processStateList.add(ProcessInstanceStateConstants.PROCESS_ACTIVE);
				processStateList.add(ProcessInstanceStateConstants.PROCESS_DONE);
				processStateList.add(ProcessInstanceStateConstants.PROCESS_WARNING);
			}
			processStateList.add(ProcessInstanceStateConstants.PROCESS_FAULTED);
			serviceResponse.setAttribute("statoprocesso", processStateList);
			List organizationList = null;
			if (((String)ConfigSingleton.getInstance().getAttribute("CONSOLE.taxonomyVisible")).equalsIgnoreCase("TRUE"))
				organizationList = ProcessBO.getAllOrganizations();
			else
				organizationList = new ArrayList();
			serviceResponse.setAttribute("orgList", organizationList);
			
			List iterList = ProcessBO.getIterList();
			serviceResponse.setAttribute("iterList", iterList);
		}	
		if (serviceRequest.getAttribute(Constants.LBL_RESTART) != null)
			serviceResponse.setAttribute(Constants.LBL_RESTART, serviceRequest.getAttribute(Constants.LBL_RESTART));
	}
	protected SourceBean createQueryBean(SearchQuery query) 
		throws SourceBeanException {		
		SourceBean item = null;
		long idProcess;
		try {
			item = new SourceBean("ITEM");
			item.setAttribute("id_query", query.getIdQuery());
			item.setAttribute("label", query.getLabel());				
			item.setAttribute("query", query.getQuery());
		} catch(SourceBeanException sbe) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessSearchAction::createQueryBean:: ", sbe);
			throw sbe;
		}
		return item;
	}

}
