/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.audit.action;

import it.eng.console.audit.bo.ProcessInstanceBO;
import it.eng.console.hibernate.util.HibernateSessionFactory;
import it.eng.console.util.Constants;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.dispatching.action.AbstractAction;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spagosoa.smx.components.restart.client.ModifiedMessage;
import it.eng.spagosoa.smx.components.restart.client.RestartServiceProxy;
import it.eng.spagosoa.smx.components.restart.client.RestartResult;

import java.rmi.RemoteException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import org.hibernate.Query;
import org.hibernate.Session;
import org.spagic.metadb.dbutils.AuditDBUtils;
import org.spagic.metadb.model.ProcessInstance;

public class RestartSingleInstanceAction extends AbstractAction {

	public void service(SourceBean serviceRequest, SourceBean serviceResponse)
			throws Exception {
		info(" Start ");
		String processId = (String)serviceRequest.getAttribute("processId");
		info(" ProcessId" + processId);
		
		String idQuery        =  ((String)serviceRequest.getAttribute(Constants.LBL_ID_QUERY) != null)
		? (String)serviceRequest.getAttribute(Constants.LBL_ID_QUERY): "";	
		
		String fromHome       =  ((String)serviceRequest.getAttribute(Constants.LBL_FROM_HOME)!=null)
		? (String)serviceRequest.getAttribute(Constants.LBL_FROM_HOME):"";
		
		String reExecuteQuery =  ((String)serviceRequest.getAttribute("reExecuteQuery") != null)
		? (String)serviceRequest.getAttribute("reExecuteQuery")	: "";
		
		long processInstanceId = Long.parseLong((String)serviceRequest.getAttribute("processInstanceId"));
		String advancedSearch = ((String)serviceRequest.getAttribute("advancedSearch") != null)
		? (String)serviceRequest.getAttribute("advancedSearch")
		: "FALSE";
		
		String key = Constants.PREFIX_LOGS + processInstanceId;
		info(" Key is " + key);
		SessionContainer sessionContainer = getRequestContainer().getSessionContainer();
		HashMap logsForInstance = null;
		if (sessionContainer.getAttribute(key) != null)
			logsForInstance = (HashMap)sessionContainer.getAttribute(key);
		
		ModifiedMessage[] modifiedMessages = generateInputForWS(logsForInstance);
		info(" Modified Message Generated");
		//call method to restart single Instance
		RestartServiceProxy proxy = new RestartServiceProxy();
		String locationRestartWS = (String)ConfigSingleton.getInstance().getAttribute("CONSOLE.locationRestartWS");
		proxy.setEndpoint(locationRestartWS);
		
		RestartResult restartResult = null;
		long newProcessInstanceId = processInstanceId;
		try {
			info(" Calling Restar Service");
			// Restart process
			restartResult = proxy.restartSingleInstance(processInstanceId, modifiedMessages);
			
			// Check for managed errors
			if (restartResult.getErrorMessage() != null) {
				info(" Restart Result has Error Message");
				EMFUserError warning = new EMFUserError(EMFErrorSeverity.INFORMATION, restartResult.getErrorMessage(), "properties.console.messages");
				getErrorHandler().addError(warning);
			} else {
				
				if ((restartResult.getRestartedInstanceId() != -1)
						&& (restartResult.getRestartedInstanceId() != processInstanceId)) {
					
					// No problems on restart
					sessionContainer.delAttribute(key);
					newProcessInstanceId = restartResult.getRestartedInstanceId();
					info(" No Problem On Restart New Process Instance Id " +restartResult.getRestartedInstanceId() );
				}
			}
		} catch(RemoteException re) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "RestartSingleInstanceAction::service:: ", re);
			EMFUserError warning = new EMFUserError(EMFErrorSeverity.INFORMATION, "10001", "properties.console.messages");
			getErrorHandler().addError(warning);
		}
		Session session = null;
		try{
			info(" Creating HQL Query ");
			String hqlQuery = "from org.spagic.metadb.model.ProcessInstance pi where pi.idProcessInstance = :p1";
			info(" Using Current Session ");
			//session = HibernateSessionFactory.getSessionFactory().openSession();
			Query q = HibernateSessionFactory.getSessionFactory().getCurrentSession().createQuery(hqlQuery);
			info(" Create And Execute HQL Query ");
			q.setLong("p1", newProcessInstanceId);
		
			ProcessInstance pi = (ProcessInstance)q.uniqueResult();
			if (pi == null){
				info("Process Instance Readed after restart is NULL");
			}else{
				info("Process Instance Readed after restart is NOT NULL"); 
			}
			info("Setting Attribute On Service Response"); 
			serviceResponse.setAttribute("processId", processId);
			serviceResponse.setAttribute("processInstanceId", Long.toString(newProcessInstanceId));
			serviceResponse.setAttribute("advancedSearch",advancedSearch);
			// 
			serviceResponse.setAttribute(Constants.LBL_ID_QUERY,idQuery);
			serviceResponse.setAttribute(Constants.LBL_FROM_HOME,fromHome);
			serviceResponse.setAttribute("reExecuteQuery",reExecuteQuery);
			//
			info("Finished Restart Action");
		}catch (Exception e) {
			e.printStackTrace();
		}finally{
			if ( ( session != null) && (session.isOpen())){
				session.close();
			}
		}

	}
	
	private ModifiedMessage[] generateInputForWS(HashMap logsForInstance){
		if (logsForInstance == null)
			return new ModifiedMessage[0];
		else {
			ModifiedMessage[] modifiedMessages = new ModifiedMessage[logsForInstance.size()];
			Set idLogSet = logsForInstance.keySet();
			Iterator iteratorLog = idLogSet.iterator();
			String idLog = null;
			String log = null;
			long idJournal;
			int index=0;
			while (iteratorLog.hasNext()) {
				idLog = (String)iteratorLog.next();
				log = (String)logsForInstance.get(idLog);
				idJournal = ProcessInstanceBO.getIdJournal(Long.parseLong(idLog));
				modifiedMessages[index++] = new ModifiedMessage(idJournal, log);
			}
			return modifiedMessages;
		}
	}
	
	
	private void info(String msg){
		TracerSingleton.log(Constants.project, TracerSingleton.INFORMATION, "RestartSingleInstanceAction -> "+ msg);
	} 
}
