/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.jmx.tag;

import it.eng.console.util.Constants;
import it.eng.jmx.bo.JmxBO;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.ResponseContainerAccess;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.message.MessageBundle;
import it.eng.spago.tracing.TracerSingleton;

import java.util.List;

import javax.management.Attribute;
import javax.management.AttributeList;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

public class QueueListTag extends TagSupport {
	private String type = null;
	private final int maxLengthName = 110;
	
    public int doStartTag() throws JspException {
        TracerSingleton.log(Constants.project, TracerSingleton.DEBUG, "QueueListTag::doStartTag:: invocato");
        if(pageContext == null) {
            TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL,
                "QueueListTag::doStartTag:: pageContext nullo");
            throw new JspException("pageContext nullo");
        } 
        HttpServletRequest httpRequest = null;
        try {
            httpRequest = (HttpServletRequest) pageContext.getRequest();
        } catch(Exception ex) {
            TracerSingleton.log(
            		Constants.project, TracerSingleton.CRITICAL, "QueueListTag::doStartTag::", ex);
            throw new JspException(ex.getMessage());
        } 
        if(httpRequest == null) {
            TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "QueueListTag::doStartTag:: httpRequest nullo");
            throw new JspException("httpRequest nullo");
        }
        ResponseContainer responseContainer =
            ResponseContainerAccess.getResponseContainer(httpRequest);
        if(responseContainer == null) {
            TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "QueueListTag::doStartTag:: httpRequest nullo");
            throw new JspException("responseContainer nullo");
        }    
        StringBuffer output = new StringBuffer(); 
        SourceBean serviceResponse = responseContainer.getServiceResponse();
		List queueList = (List)serviceResponse.getAttribute(type);
		if (queueList == null) {
			if(type.equals(it.eng.jmx.util.Constants.LBL_TOPIC)) 
				output.append("<table id=\"topicTable\">\n");
			else
				output.append("<table id=\"queueTable\">\n");
			output.append("<thead>\n");
			output.append("</thead>\n");
			output.append("<tbody class=\"scrollingContent\">");
			output.append("<tr>\n");
			output.append("<td>"+ MessageBundle.getMessage("smxNotAvailable", "properties.spagic") + "</td>\n"); 
			output.append("</tr>\n");
			output.append("</tbody>\n");
			output.append("</table>\n");
		} else {
			if(type.equals(it.eng.jmx.util.Constants.LBL_TOPIC)) 
				output.append("<table id=\"topicTable\">\n");	
			else 
				output.append("<table id=\"queueTable\">\n");	
			AttributeList attributeValues;
			int indexList;
			
			//intestazione colonne
			ConfigSingleton config = ConfigSingleton.getInstance();
//			String idDefaultServer= (String)config.getAttribute("SERVERS.default");
			String[] attributeNames = {"Name","MemoryPercentUsage","QueueSize","EnqueueCount","DispatchCount","DequeueCount","ConsumerCount"};
			
			output.append("<thead>\n");
			output.append("<tr>\n");

			for(int index = 0; index < attributeNames.length; index++) {
				output.append("<td>"); 
				output.append(attributeNames[index] + "</td>\n");
			}
			output.append("</tr>\n");
			output.append("</thead>\n");
			
			//contenuto righe tabella
			output.append("<tbody class=\"scrollingContent\">");	
			
			for(indexList=0; indexList<queueList.size(); indexList++) {
				attributeValues = (AttributeList)queueList.get(indexList);
				output.append("<tr>\n");
				
				for(int jj = 0; jj < attributeNames.length; jj++) {
					renderAttribute(getAttributeByName(attributeValues, attributeNames[jj]), output);
				}
				
				output.append("</tr>\n");
			}
			output.append("</tbody>");
			output.append("</table>\n");
		}
        try {
            pageContext.getOut().print(output.toString());
        } catch(Exception ex) {
            TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "QueueListTag::doStartTag::", ex);
            throw new JspException(ex.getMessage());
        } 
        return SKIP_BODY;
    }
    
    private Attribute getAttributeByName(AttributeList list, String name) {
    	Attribute result = null;
		for (Object attr : list) {
			if (name.equalsIgnoreCase(((Attribute)attr).getName())) {
				result = (Attribute)attr;
				break;
			}
		}
		return result;
    }
    
    private void renderAttribute(Attribute attr, StringBuffer output) {
		String attributeValue = (attr != null) ? attr.getValue().toString() : "";
		if (attributeValue.length() > maxLengthName) 
			attributeValue = attributeValue.substring(0, maxLengthName);
		output.append("<td>" + attributeValue + "</td>\n"); 
    }
    
	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	/**
     * @see javax.servlet.jsp.tagext.Tag#doEndTag()
     */
    public int doEndTag() throws JspException {
        TracerSingleton.log(Constants.project, TracerSingleton.DEBUG, "QueueListTag::doEndTag:: invocato");
        return super.doEndTag();
    } 

}
